#!/usr/bin/env python

################################
# gdgen 0.1
#################################
# copyright 2005 Thomas Bell
# released under the GPL
# see a copy of the license at http://www.gnu.org/copyleft/gpl.html
##################################
# requirements: 
# dgen, pygtk
##################################
# use: 
# python gdgen.py
##################################
import sys, os, time, string, threading
import pygtk, gtk, gtk.glade, gobject
import fileinput

USER_NAME = os.getlogin()
PREFS_DIR = '/home/'+USER_NAME+'/.gdgen/'


class DgenGui:
    def __init__(self):
        self.wTree = gtk.glade.XML("gdgen.glade")

        dic = {
        "gtk_main_quit" : self.delete_event,
        "on_open1_activate" : self.create_fileselect,
        "on_preferences1_activate" : self.create_prefs,
        "on_preferences_cancel_clicked" : self.hide_prefs,
        "on_preferences_apply_clicked" : self.write_prefs,
        "on_preferences_ok_clicked" : self.write_prefs,
        "on_preferences_ok_clicked" : self.hide_prefs,
        "on_cancel_button_clicked" : self.hide_fileselect,
        "on_open_button_clicked" : self.open_clicked,
        "on_quit1_activate" : self.delete_event,
        "on_quit_button_clicked" : self.delete_event,
        "on_run_button_clicked" : self.run_clicked,
        "on_rm_button_clicked" : self.rm_clicked,
        "on_filechooserbutton1_current_folder_changed" : self.set_rompath,
        "on_entry1_changed" : self.exe_default
        }
        self.wTree.signal_autoconnect(dic)
        
        self.options = {}
        self.prefs = self.wTree.get_widget("preferenceswindow1")
        self.dgen_entry = self.wTree.get_widget("entry1")
        self.rompath_entry = self.wTree.get_widget("entry2")
        self.fullscreen_cb = self.wTree.get_widget("fullscreen_checkbutton")
        self.joystick_cb = self.wTree.get_widget("joystick_checkbutton")
        self.pal_cb = self.wTree.get_widget("pal_checkbutton")
        
        self.create_treeview()
        self.read_prefs()
        
        s = self.rompath_entry.get_text()
        self.add_roms(s)
        
        return

    def main(self):
        gtk.main()


    def start(self):
        try:
            self.main()
        except KeyboardInterrupt:
            self.on_keyboard_interrupt()

    def set_rompath(self, widget):
        self.filechooser_button = self.wTree.get_widget("filechooserbutton1")
        self.rompath_entry.set_text(self.filechooser_button.get_filename())

    def exe_default(self, widget):
        if self.dgen_entry.get_text() == None:
            self.dgen_entry.set_text('dgen')
        return


    def create_prefs(self, widget):
        self.prefs.show()

    def hide_prefs(self, widget):
        self.prefs.hide()

    def create_fileselect(self, widget):
        self.filew = self.wTree.get_widget("filechooserdialog1")
        self.filew.show()

    def hide_fileselect(self, widget):
        self.filew.hide()

    def open_clicked(self, widget):
        src = self.filew.get_filename()
        self.filew.hide()
        self.add_roms(src)

    def delete_event(self, widget):
        gtk.main_quit()

    def create_treeview(self):
        self.treeview = self.wTree.get_widget("treeview1")
        self.treemodel = gtk.TreeStore('gchararray',
                'gchararray', 'gchararray')
        self.treeview.set_model(self.treemodel)
        self.treeview.set_headers_visible(True) 
        renderer=gtk.CellRendererText()
        
        column1=gtk.TreeViewColumn("rom",renderer, text=0)
        column1.set_resizable(True)
        self.treeview.append_column(column1)
        
        column2=gtk.TreeViewColumn("path",renderer, text=1)
        column2.set_resizable(True)
        self.treeview.append_column(column2)

        self.treeview.add_events(gtk.gdk.BUTTON_PRESS_MASK)
        self.treeview.connect('button_press_event', self.rom_clicked)

        self.treeview.show()

    def insert_row(self, parent, column1, column2):
        myiter = self.treemodel.insert_after(parent,None)
        self.treemodel.set_value(myiter,0,column1)
        self.treemodel.set_value(myiter,1,column2)
        return myiter

    def add_roms(self, src):
        for root, dirs, files in os.walk(src):
            for f in files:
                ext = string.lower(os.path.splitext(f)[1])
                if ext == '.smd' or ext == '.bin':
                    fullpath = root + '/' + f
                    self.insert_row(None,f,fullpath)
                    print root
                    print f
                else:
                    print '%s %s' % (f, 'is not a rom!')

    def run_rom(self, fullpath_rom):
        self.dgen = Dgen()
        self.dgen.run_rom(self.get_prefs()[0], fullpath_rom,
                self.get_prefs()[2], self.get_prefs()[3], 
                self.get_prefs()[4])


    def rom_clicked(self, widget, event):
        if event.button == 1 and event.type == gtk.gdk._2BUTTON_PRESS:
            selection = self.treeview.get_selection()
            selected, iter = selection.get_selected()
            item = {}
            item['Rom'] = selected.get_value(iter,0)
            item['Rompath'] = selected.get_value(iter,1)
            print 'running %s' % item['Rom']

            self.run_rom(item['Rompath'])

    def run_clicked(self, widget):
        print 'run button clicked'
        selection = self.treeview.get_selection()
        selected, iter = selection.get_selected()
        item = {}
        item['Rom'] = selected.get_value(iter,0)
        item['Rompath'] = selected.get_value(iter,1)
        print 'running %s' % item['Rompath']

        self.run_rom(item['Rom'])
        

    def rm_clicked(self, widget):
        print 'rm button clicked'

    def get_prefs(self):
        #fix
        #exe = self.dgen_entry.get_text()
        exe = 'dgen'
        rompath = self.rompath_entry.get_text()
        
        if self.fullscreen_cb.get_active():
            fullscreen = '-f'
        else:
            fullscreen = ''

        if self.joystick_cb.get_active():
            joystick = '-j'
        else:
            joystick = ''

        if self.pal_cb.get_active():
            pal = '-P'
        else:
            pal = ''
            
        return exe, rompath, fullscreen, joystick, pal

    def write_prefs(self, widget):
        print 'apply or ok clicked'

        if os.path.exists(PREFS_DIR):
            self.prefs_file = open(PREFS_DIR + 'prefs.txt', 'w')
            self.prefs_file.write('fullscreen=' + self.get_prefs()[2] + '\n')
            self.prefs_file.write('joystick=' + self.get_prefs()[3] + '\n')
            self.prefs_file.write('PAL=' + self.get_prefs()[4] + '\n')
            self.prefs_file.write('exe=' + self.get_prefs()[0] + '\n')
            self.prefs_file.write('rompath=' + self.get_prefs()[1] + '\n')
            self.prefs_file.close()
        else:
            os.mkdir(PREFS_DIR)
            self.prefs_file = open(PREFS_DIR + 'prefs.txt', 'w')
            self.prefs_file.write('fullscreen=' + self.get_prefs()[2] + '\n')
            self.prefs_file.write('joystick=' + self.get_prefs()[3] + '\n')
            self.prefs_file.write('PAL=' + self.get_prefs()[4] + '\n')
            self.prefs_file.write('exe=' + self.get_prefs()[0] + '\n')
            self.prefs_file.write('rompath=' + self.get_prefs()[1] + '\n')
            self.prefs_file.close()

        print 'prefs file written'

    def read_prefs(self):
        print 'reading config file'
        if os.path.exists(PREFS_DIR):
            #self.prefs_file = open(PREFS_DIR + 'prefs.txt', 'r')
            prefs_path = PREFS_DIR + 'prefs.txt'

            lines = []
            for line in fileinput.input(prefs_path):
                lines.append(line)

            
            if lines[0][11:][:2] == '-f':
                self.fullscreen_cb.set_active(True)
            else:
                self.fullscreen_cb.set_active(False)

            if lines[1][9:][:2] == '-j':
                self.joystick_cb.set_active(True)
            else:
                self.joystick_cb.set_active(False)

            if lines[2][4:][:2] == '-P':
                self.pal_cb.set_active(True)
            else:
                self.pal_cb.set_active(False)

            if lines[3][4:][:2] == '':
                self.dgen_entry.set_text('dgen')
            else:
                self.dgen_entry.set_text(lines[3][4:])

            self.rompath_entry.set_text(lines[4][8:])

        else:
            return
        



class Dgen(threading.Thread):
    def __init__(self):
        threading.Thread.__init__(self)

    def run(self):
        print 'starting thread'
        print 'running %s' % self.rom
        run_string = self.exe + ' ' +self.fs+ ' ' +self.js+ ' ' +self.pal+ ' "' +self.rom + '"'
        os.system(run_string)

    def run_rom(self, exe, rom, fs, js, pal):
        self.exe = exe
        self.rom = rom
        self.fs = fs
        self.js = js
        self.pal = pal
        self.start()
        

gtk.gdk.threads_init()
app = DgenGui()
gtk.gdk.threads_enter()
app.start()
gtk.gdk.threads_leave()
